/*
 * DbDefenition.java
 *
 * Created on 22  2002 ., 12:27
 */

package com.fn.dbgen;


import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.FactoryConfigurationError;  
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;  
import org.xml.sax.SAXParseException;
import org.w3c.dom.Document;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author  asso
 */
public class DbDefenition {

    private Map fieldsDefs;
    private PrimaryDef primaryDef;
    private UniqueDef uniqueDef;
    private IndexDef indexDef;
    private ForeignDef foreignDef;
    private String name;
    private TableDef tableDef;
    private InsertDef insertDef;
    private boolean allowNull;
    
    /** Creates a new instance of DbDefenition */
    public DbDefenition(String fileName) throws DgException {
        fieldsDefs = new HashMap();
        parse(fileName);
    }
    
    private void parse(String fileName) throws DgException {
        Document document;
        
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setIgnoringComments(true);
            factory.setIgnoringElementContentWhitespace(true);
            DocumentBuilder builder = factory.newDocumentBuilder();
            document = builder.parse(new File(fileName));
        } catch (SAXException sxe) {
            Exception x = sxe;
            if (sxe.getException() != null)
                x = sxe.getException();
            throw new DgException("Error parsing defenition: " + x.getMessage());
        } catch (ParserConfigurationException pce) {
            throw new DgException("Parser with specified options can't be built");
        } catch (IOException ioe) {
            throw new DgException("Error reading defenition '" + fileName + "'");
        } catch (Exception e) {
            throw new DgException("Unknown exception in defenition parser: " + e.getMessage());
        }

        Element root = document.getDocumentElement();
        if (! root.getNodeName().equals("definition"))
            throw new DgException("<definition> expected but '" + root.getNodeName() + "' found");
        name = root.getAttribute("name");
        allowNull = XMLUtils.getTagAttrib(root, "null", true);
        
        NodeList nodeList = root.getChildNodes();
        int len = nodeList.getLength();
        for (int i = 0; i < len; i++) {
            Node node = nodeList.item(i);
            switch (node.getNodeType()) {
                case Node.ELEMENT_NODE:
                    processElement((Element)node);
                    break;            
                case Node.TEXT_NODE:
                    if (XMLUtils.isBlankText(node.getNodeValue()))
                        break;                    
                default:
                    throw new DgException("Error in definition file: unexpected element");
            }
        }
    }
    
    private void processElement(Element element) throws DgException {
        String name = element.getNodeName();
        if (name.equals("fieldtype")) {
            FieldDef fd = new FieldDef(element);
            fieldsDefs.put(fd.getType(), fd);
        } else if (name.equals("primary"))
            primaryDef = new PrimaryDef(element);
        else if (name.equals("unique"))
            uniqueDef = new UniqueDef(element);
        else if (name.equals("index"))
            indexDef = new IndexDef(element);
        else if (name.equals("foreign"))
            foreignDef = new ForeignDef(element);
        else if (name.equals("table"))
            tableDef = new TableDef(element);
        else if (name.equals("insert"))
            insertDef = new InsertDef(element);
        else
            throw new DgException("Invalid definition element '" + name + "'");
    }
    
    public FieldDef getFieldDef(String name) {
        return (FieldDef)fieldsDefs.get(name);
    }
    
    public PrimaryDef getPrimaryDef() {
        return primaryDef;
    }
    
    public UniqueDef getUniqueDef() {
        return uniqueDef;
    }
    
    public IndexDef getIndexDef() {
        return indexDef;
    }
    
    public ForeignDef getForeignDef() {
        return foreignDef;
    }
    
    public String getName() {
        return name;
    }
    
    public TableDef getTableDef() {
        return tableDef;
    }
    
    public InsertDef getInsertDef() {
        return insertDef;
    }
    
    public boolean getNull() {
        return allowNull;
    }
}
