/*
 * FieldDef.java
 *
 * Created on 22  2002 ., 13:13
 */

package com.fn.dbgen;


import java.util.HashMap;
import java.util.Map;
import org.w3c.dom.Element;


/**
 *
 * @author  asso
 */
public class FieldDef {

    private String type;
    private String sqltype;
    private String beforeTable;
    private String afterTable;

    /** Creates a new instance of FieldDef */
    public FieldDef(Element element) throws DgException {
        type = element.getAttribute("name");
        if (type.equals(""))
            throw new DgException("Invalid field type");
        Element subTag = XMLUtils.getSubtag(element, "sqlname");
        if (subTag != null)
            sqltype = XMLUtils.getTrimmedData(subTag);
        if ((subTag == null) || sqltype.equals(""))
            throw new DgException("Invalid sqlname of fieldtype '" + type + "'");
        subTag = XMLUtils.getSubtag(element, "beforetable");
        if (subTag != null)
            beforeTable = XMLUtils.getTrimmedData(subTag);
        subTag = XMLUtils.getSubtag(element, "aftertable");
        if (subTag != null)
            afterTable = XMLUtils.getTrimmedData(subTag);
    }

    public String getType() {
        return type;
    }
    
    private Map createMacrosArguments(Table table) {
        Map map = new HashMap();
        map.put("t", table.getName());
        return map;
    }
    
    public void generateField(Element field, Table table, DbDefenition def) throws DgException {
        boolean allowNull = XMLUtils.getTagAttrib(field, "null", true);
        String name = XMLUtils.getTrimmedData(field);
        Map args = createMacrosArguments(table);
        args.put("f", name);
        if (allowNull) {
            if (def.getNull())
                args.put("n", "NULL");
            else
                args.put("n", "");
        } else
            args.put("n", "NOT NULL");
        table.addDef(Macros.processMacroses(sqltype, field, args));
        if ((beforeTable != null) && (beforeTable.length() > 0))
            table.addBefore(Macros.processMacroses(beforeTable, field, args));
        if ((afterTable != null) && (afterTable.length() > 0))
            table.addAfter(Macros.processMacroses(afterTable, field, args));
    }    
}
